@@ -10,22 +10,21 @@
 #define _BIRD_BIRDLIB_H_
 
 #include "lib/alloca.h"
+#include <stddef.h>
 #include <stdarg.h>
+#include <stdalign.h>
 
 /* Ugly structure offset handling macros */
 
-struct align_probe { char x; long int y; };
-
 #define OFFSETOF(s, i) ((size_t) &((s *)0)->i)
 #define SKIP_BACK(s, i, p) ((s *)((char *)p - OFFSETOF(s, i)))
 #define BIRD_ALIGN(s, a) (((s)+a-1)&~(a-1))
-#define CPU_STRUCT_ALIGN (sizeof(struct align_probe))
+#define CPU_STRUCT_ALIGN (alignof(max_align_t))
 
 /* Utility macros */
 
 #define MIN_(a,b) (((a)<(b))?(a):(b))
 #define MAX_(a,b) (((a)>(b))?(a):(b))
-#define CLAMP(a,l,h) (((a)<(l)) ? (l) : (((a)>(h)) ? (h) : (a)))
 
 #ifndef PARSER
 #undef MIN
