@@ -690,7 +690,7 @@ static void
 bgp_format_cluster_list(const eattr *a, byte *buf, uint size)
 {
   /* Truncates cluster lists larger than buflen, probably not a problem */
-  int_set_format(a->u.ptr, 0, -1, buf, size);
+  int_set_format(a->u.ptr, ISF_ROUTER_ID, -1, buf, size);
 }
 
 
@@ -1247,7 +1247,11 @@ bgp_export_attrs(struct bgp_export_state *s, ea_list *attrs)
 static inline int
 bgp_encode_attr(struct bgp_write_state *s, eattr *a, byte *buf, uint size)
 {
-  ASSERT(EA_PROTO(a->id) == PROTOCOL_BGP);
+  if (EA_PROTO(a->id) != PROTOCOL_BGP)
+    if (s->ignore_non_bgp_attrs)
+      return 0;
+    else
+      bug("Tried to encode a non-BGP attribute");
 
   uint code = EA_ID(a->id);
 
